package com.ebei.device.asset.weixin.auth;

import com.ebei.device.asset.weixin.exception.LoginException;
import com.ebei.device.asset.weixin.service.UserSessionManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Enumeration;

@Slf4j
public class LoginFilter  extends HandlerInterceptorAdapter {

    @Autowired
	UserSessionManager userSessionManager;
    @Override  
    public boolean preHandle(HttpServletRequest request,
                             HttpServletResponse response, Object handler) throws Exception {
        log.debug("==============Pre LoginFilter================");
    	String tokenValue = parseToken(request);
        String method = request.getMethod();
        if("OPTIONS".equals(method)){
            return true;
        }
         log.debug("token is {}",tokenValue);
		if (tokenValue== null ) {
			 log.warn("用户未登录,token is null",tokenValue);
	         throw new LoginException("请登录之后操作");
		}
		AuthUser user  = userSessionManager.getAuthUser(tokenValue);
		if(user==null) {
			log.warn("用户信息失效,{}",tokenValue);
			throw new LoginException("用户信息失效,请登录之后操作");
		}
		request.setAttribute(AuthUtils.REQ_USER, user);
		request.setAttribute(AuthUtils.REQ_TOKEN, tokenValue);
		return true;
           

    } 
	protected String parseToken(HttpServletRequest request) {
		// first check the header...
		String token = parseHeaderToken(request);
		// bearer type allows a request parameter as well
		if (token == null) {
			token = request.getParameter(ACCESS_TOKEN);
			request.setAttribute(AuthUtils.REQ_BLANK, true);
		}
		return token;
	}
	@SuppressWarnings("unchecked")
	protected String parseHeaderToken(HttpServletRequest request) {
		Enumeration<String> headers = request.getHeaders("Authorization");
		while (headers.hasMoreElements()) { // typically there is only one (most servers enforce that)
			String value = headers.nextElement();
			if ((value.toLowerCase().startsWith(BEARER_TYPE.toLowerCase()))) {
				String authHeaderValue = value.substring(BEARER_TYPE.length()).trim();
				int commaIndex = authHeaderValue.indexOf(',');
				if (commaIndex > 0) {
					authHeaderValue = authHeaderValue.substring(0, commaIndex);
				}
				return authHeaderValue;
			}
			else {
				// todo: support additional authorization schemes for different token types, e.g. "MAC" specified by
				// http://tools.ietf.org/html/draft-hammer-oauth-v2-mac-token
			}
		}
		return request.getHeader("token");
	}
	public static final String BEARER_TYPE = "Bearer";
	public static final String ACCESS_TOKEN = "access_token";
	
	public static void main(String[] args) {
		AntPathMatcher antMatcher = new AntPathMatcher();
		String url = "/api/v1/file/app/app_12.apk";
		System.out.println(antMatcher.match("/**/file/**/*", url));
		url = "/device/address/getAddressQrCode/1210104410705780738";
		System.out.println(antMatcher.match("**address/getAddressQrCode**", url));
	}
}
