package com.ebei.device.asset.weixin.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

/**
 * @Description: 通用线程池配置
 * @Auther: yangsm
 * @Date: 2019/3/27 0027
 */
@Configuration
@EnableAsync
public class TaskThreadPoolConfig {
    @Value("${task.corePoolSize}")
    private int corePoolSize;//设置核心线程数
    @Value("${task.maxPoolSize}")
    private int maxPoolSize;//设置最大线程数
    @Value("${task.queueCapacity}")
    private int queueCapacity; //线程池所使用的缓冲队列

    @Bean(name = "taskExecutor")
    public ThreadPoolTaskExecutor taskExecutor() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(corePoolSize);
        executor.setMaxPoolSize(maxPoolSize);
        executor.setQueueCapacity(queueCapacity);
        //等待任务在关机时完成--表明等待所有线程执行完
        executor.setWaitForTasksToCompleteOnShutdown(true);
        // 等待时间 （默认为0，此时立即停止），并没等待xx秒后强制停止
        executor.setAwaitTerminationSeconds(60);
        //  线程名称前缀
        executor.setThreadNamePrefix("time-task-");
        executor.initialize();
        return executor;
    }
}
