package com.ebei.device.asset.weixin.config;

import com.ebei.device.asset.weixin.auth.LoginFilter;
import com.ebei.device.asset.weixin.constant.Constants;
import com.ebei.device.asset.weixin.util.FileUploadDownloadUtil;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.web.servlet.MultipartConfigFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.util.unit.DataSize;
import org.springframework.util.unit.DataUnit;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.config.annotation.InterceptorRegistration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import javax.servlet.MultipartConfigElement;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.List;


/**
 * <p>
 * WEB 初始化相关配置
 * </p>
 *
 * @author hubin
 * @since 2017-12-11
 */
@ControllerAdvice
@Configuration
public class WebConfig extends WebMvcConfigurerAdapter {
	@Autowired
	CommProperties commProperties;
	
	@Bean
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public ObjectMapper objectMapper() {
		Jackson2ObjectMapperBuilder builder = new Jackson2ObjectMapperBuilder();
        builder.serializationInclusion(JsonInclude.Include.ALWAYS);
        ObjectMapper objectMapper = builder.build();
        SimpleModule simpleModule = new SimpleModule();
        simpleModule.addSerializer(Long.class, ToStringSerializer.instance);
        objectMapper.registerModule(simpleModule);
        objectMapper.configure(MapperFeature.PROPAGATE_TRANSIENT_MARKER, true);// 忽略 transient 修饰的属性
        objectMapper.setDateFormat(new SimpleDateFormat(Constants.DEFAULT_DATE_FORMATE));
        return objectMapper;
	}

    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
        Jackson2ObjectMapperBuilder builder = new Jackson2ObjectMapperBuilder();
        builder.serializationInclusion(JsonInclude.Include.ALWAYS);
        ObjectMapper objectMapper = builder.build();
        SimpleModule simpleModule = new SimpleModule();
        simpleModule.addSerializer(Long.class, ToStringSerializer.instance);
        objectMapper.registerModule(simpleModule);
        objectMapper.configure(MapperFeature.PROPAGATE_TRANSIENT_MARKER, true);// 忽略 transient 修饰的属性
        objectMapper.setDateFormat(new SimpleDateFormat(Constants.DEFAULT_DATE_FORMATE));
        converters.add(new MappingJackson2HttpMessageConverter(objectMapper));
        super.configureMessageConverters(converters);
    }
    
	public void addInterceptors(InterceptorRegistry registry) {
		 InterceptorRegistration interceptor = registry.addInterceptor(loginFilter());
	   /**
	    * 添加拦截的路径
        * /为根路径
        * /*为一级路径
        * /** 为所有路径包括多级
        */
	   interceptor.addPathPatterns("/**");
		//排除不拦截的，包括自己登录的页面不用拦截
	   interceptor.excludePathPatterns(commProperties.getWhiteList());
       
	}
	
    @Bean
    public LoginFilter loginFilter() {
    	LoginFilter loginFilter = new LoginFilter();
    	return loginFilter;
    }
    

    /**
     * 文件上传配置
     * @return
     *      
     */
    @Bean
    public MultipartConfigElement multipartConfigElement() {
    	
        MultipartConfigFactory factory = new MultipartConfigFactory();
        //文件最大
        factory.setMaxFileSize(DataSize.of(20480, DataUnit.KILOBYTES));//KB,MB
//        factory.setMaxFileSize("20480KB");
        // 设置总上传数据总大小exportBatchAddTemplate
        factory.setMaxRequestSize(DataSize.of(20480, DataUnit.KILOBYTES));
        File fileExist = new File(commProperties.getFileRoot()+"/temp");
//        System.out.println(">>>>>>>>>>>>>>>>>"+fileExist.getAbsolutePath());
        FileUploadDownloadUtil.cleanDir(fileExist);
        if(!fileExist.exists()){
            fileExist.mkdirs();
        }
        factory.setLocation(fileExist.getAbsolutePath());
        return factory.createMultipartConfig();
    }
}
