
package com.ebei.device.asset.weixin.contorller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.mapper.Wrapper;
import com.ebei.device.asset.weixin.auth.AuthUtils;
import com.ebei.device.asset.weixin.model.entity.AssetTenantUser;
import com.ebei.device.asset.weixin.service.AssetTenantUserService;
import com.ebei.device.asset.weixin.service.UserSessionManager;
import com.ebei.device.asset.weixin.util.MD5;
import com.ebei.device.asset.weixin.util.ResponseEx;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * auto code version 1.3
 * <p>
 * 登录
 * </p>
 * 
 * @author yangsm
 * @since 2018-08-20
 */
@RestController
@RequestMapping("/login")
@Api(value = "登陆信息", description = "登陆信息")
public class LoginController {
	@Autowired
	private AssetTenantUserService assetTenantUserService;
	@Autowired
	private UserSessionManager userSessionManager;

	@ApiOperation(value = "登陆校验", notes = "登陆校验")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userAccount", value = "登录名", required = true, paramType = "query"),
			@ApiImplicitParam(name = "password", value = "密码", required = true, paramType = "query") })
	@GetMapping("getCheckedLoginInfo")
	public ResponseEx<Object> getCheckedLoginInfo(@RequestParam String userAccount, @RequestParam String password) {
		return postCheckedLoginInfo(userAccount, password);
	}

	@ApiOperation(value = "安全登陆校验", notes = "安全登陆校验")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userAccount", value = "登录名", required = true, paramType = "query"),
			@ApiImplicitParam(name = "password", value = "密码", required = true, paramType = "query") })
	@PostMapping("postCheckedLoginInfo")
	public ResponseEx<Object> postCheckedLoginInfo(@RequestParam String userAccount, @RequestParam String password) {

		ResponseEx<Object> resultResponse = ResponseEx.createError("登录失败");
		if ((userAccount != null && password != null) && (!"".equals(userAccount) && !"".equals(password))) {
			Wrapper wrapper = new EntityWrapper(new AssetTenantUser(userAccount));
			AssetTenantUser assetTenantUser = assetTenantUserService.selectOne(wrapper);


			/*校验用户登录信息-------------------开始*/
			boolean isOk = false;
			if (null != assetTenantUser) {
				String pwd = assetTenantUser.getPwd();
				String md5comparePwd = new MD5().getMD5ofStr(password);
				if (null != pwd) {
					if (pwd.equals(password) || pwd.equals(md5comparePwd)) {
						isOk = true;
					}
				}
			}
			/*校验用户登录信息-------------------结束*/
			Map<String, Object> result = new HashMap<String, Object>();
			if (isOk) {
				Long companyId = assetTenantUser.getCompanyId();
				if (null != companyId) {
					result.put("userId", assetTenantUser.getUserId());
					result.put("companyId", assetTenantUser.getCompanyId());
					result.put("userName", assetTenantUser.getUserName());
					result.put("token", userSessionManager.createToken(assetTenantUser, companyId));
				} else {
					resultResponse = ResponseEx.createError("无效的公司");
				}
				resultResponse = ResponseEx.createSuccess("登录成功", result);
			} else {
				resultResponse = ResponseEx.createError("密码错误");
			}
		} else {
			resultResponse = ResponseEx.createError("登陆失败，用户名或密码错误！");
		}
		return resultResponse;
	}

	@ApiOperation(value = "登出", notes = "登出")
	@GetMapping("logout")
	public ResponseEx<Object> postCheckedLoginInfo(HttpServletRequest request) {
		ResponseEx<Object> resultResponse = ResponseEx.createSuccess("成功登出");
		userSessionManager.clearToken(request.getAttribute(AuthUtils.REQ_TOKEN).toString(),true);
		return resultResponse;
	}
	
	@ApiOperation(value = "登出当前用户的token缓存", notes = "登出当前用户的token缓存")
	@GetMapping("clearToken")
	public ResponseEx<Object> clearToken(@RequestParam(required = false)  String userAccount, @RequestParam(required = false) String token) {
		ResponseEx<Object> resultResponse = ResponseEx.createSuccess("成功登出");
		if(StringUtils.isNotBlank(token)) {
			userSessionManager.clearToken(token,false);
		}else if(StringUtils.isNotBlank(userAccount)) {
			AssetTenantUser user = new AssetTenantUser(userAccount);
			Wrapper<AssetTenantUser> wrapper = new EntityWrapper<>(user);
			user = assetTenantUserService.selectOne(wrapper);
			if(user!=null) {
				userSessionManager.clearToken(user.getToken(),false);
			}else {
				resultResponse.setMessage("请输入账号");
			}
		}else {
			resultResponse.setMessage("请输入账号");
		}
		return resultResponse;
	}


	@ApiOperation(value = "修改密码", notes = "修改密码")
	@GetMapping("updatePassword")
	public ResponseEx<Object> updatePassword(@RequestParam String userAccount, @RequestParam String oldPassword, @RequestParam String newPassword) {

		AssetTenantUser assetTenantUser = new AssetTenantUser(userAccount);
		Wrapper<AssetTenantUser> wrapper = new EntityWrapper<>(assetTenantUser);
		assetTenantUser = assetTenantUserService.selectOne(wrapper);
		boolean isOk = false;
		String pwd = assetTenantUser.getPwd();
		String md5comparePwd = new MD5().getMD5ofStr(oldPassword);
		if (null != pwd) {
			if (pwd.equals(oldPassword) || pwd.equals(md5comparePwd)) {
				isOk = true;
			}
		}
		if (isOk){
			userSessionManager.clearToken(assetTenantUser.getToken(),true);

			assetTenantUser.setPwd(newPassword);
			assetTenantUser.setToken(null);
			assetTenantUserService.updateById(assetTenantUser);

			return ResponseEx.createSuccess("修改成功");
		}else {
			return ResponseEx.createError("登陆失败，用户名或密码错误！");
		}

	}
}
