package com.ebei.device.asset.weixin.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.ebei.device.asset.weixin.auth.AuthUtils;
import com.ebei.device.asset.weixin.exception.BaseException;
import com.ebei.device.asset.weixin.mapper.AssetProposalFileMapper;
import com.ebei.device.asset.weixin.mapper.AssetProposalMapper;
import com.ebei.device.asset.weixin.model.entity.AssetProposal;
import com.ebei.device.asset.weixin.model.entity.AssetProposalFile;
import com.ebei.device.asset.weixin.service.AssetProposalService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * <p>
 * 资产管护租户建议反馈 服务实现类
 * </p>
 *
 */
@Slf4j
@Service
public class AssetProposalServiceImpl extends ServiceImpl<AssetProposalMapper, AssetProposal> implements AssetProposalService {

    @Autowired
    private AssetProposalFileMapper assetProposalFileMapper;

    @Override
    public AssetProposal saveAssetProposal(AssetProposal assetProposal) {

        Date date = new Date();
        //保存建议反馈
        assetProposal.setCompanyId(AuthUtils.getCompanyId());
        assetProposal.setCreator(AuthUtils.getUserId());
        assetProposal.setCreateDate(date);
        this.baseMapper.insert(assetProposal);

        if(assetProposal.getFileList() != null && assetProposal.getFileList().size()>0){
            for(AssetProposalFile file : assetProposal.getFileList()){
                file.setProposalId(assetProposal.getProposalId());
                file.setCreateTime(date);
                assetProposalFileMapper.insert(file);
            }
        }
        return assetProposal;
    }

    @Override
    public Page<AssetProposal> selectAssetProposalList(Page<AssetProposal> page, Map<String, Object> params) {

        params.put("companyId",AuthUtils.getCompanyId());
        List<AssetProposal> list = this.baseMapper.selectAssetProposalList(page, params);

        if(list != null && list.size()>0){
            for(AssetProposal assetProposal : list){
                EntityWrapper<AssetProposalFile> wp = new EntityWrapper<>();
                wp.eq("proposal_id",assetProposal.getProposalId());
                List<AssetProposalFile> fileList = assetProposalFileMapper.selectList(wp);
                if(fileList != null && fileList.size()>0){
                    assetProposal.setFileList(fileList);
                }
            }
        }

        return page.setRecords(list);
    }

    @Override
    public void updateProposalState(Long proposalId) {
        AssetProposal assetProposal = this.baseMapper.selectById(proposalId);
        if(assetProposal == null){
            throw new BaseException("建议反馈不存在");
        }

        AssetProposal updateProposal = new AssetProposal();
        updateProposal.setProposalId(proposalId);
        updateProposal.setProposalState(AssetProposal.PROPOSAL_STATE_YES);
        updateProposal.setOperator(AuthUtils.getUserId());
        updateProposal.setOperateDate(new Date());
        this.baseMapper.updateById(updateProposal);
    }

}
