package com.ebei.device.asset.weixin.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.ebei.device.asset.weixin.auth.AuthUtils;
import com.ebei.device.asset.weixin.exception.BaseException;
import com.ebei.device.asset.weixin.mapper.AssetTenantUserMapper;
import com.ebei.device.asset.weixin.mapper.AssetUserMapper;
import com.ebei.device.asset.weixin.model.entity.AssetTenantUser;
import com.ebei.device.asset.weixin.model.entity.AssetUser;
import com.ebei.device.asset.weixin.service.AssetTenantUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * <p>
 * 资产管护租户账号 服务实现类
 * </p>
 *
 */
@Slf4j
@Service
public class AssetTenantUserServiceImpl extends ServiceImpl<AssetTenantUserMapper, AssetTenantUser> implements AssetTenantUserService {


    @Autowired
    private AssetUserMapper assetUserMapper;


    @Override
    public AssetTenantUser saveAssetTenantUser(AssetTenantUser assetTenantUser) {

        if(assetTenantUser.getTenantId() == null){
            throw new BaseException("缺少租户信息");
        }

        if(assetTenantUser.getAssetList() == null || assetTenantUser.getAssetList().size()<0){
            throw new BaseException("缺少资产信息");
        }

        if(assetTenantUser.getUserAccount() == null){
            throw new BaseException("账号为必填项");
        }

        if(assetTenantUser.getPwd() == null){
            throw new BaseException("密码为必填项");
        }

        //判断账号是否唯一
        EntityWrapper<AssetTenantUser> wrapper = new EntityWrapper<>();
        wrapper.eq("company_id", AuthUtils.getCompanyId());
        wrapper.eq("user_account",assetTenantUser.getUserAccount());
        int num = this.baseMapper.selectCount(wrapper);
        if(num>0){
            throw new BaseException("账号已存在");
        }

        assetTenantUser.setCompanyId(AuthUtils.getCompanyId());
        assetTenantUser.setCreator(AuthUtils.getUserId());
        assetTenantUser.setCreateDate(new Date());

        this.baseMapper.insert(assetTenantUser);

        //保存账号与资产的关联关系
        for(AssetUser assetUser : assetTenantUser.getAssetList()){
            assetUser.setUserId(assetTenantUser.getUserId());
            assetUserMapper.insert(assetUser);
        }
        return assetTenantUser;
    }

    @Override
    public Page<AssetTenantUser> selectAssetTenantUserList(Page<AssetTenantUser> page, Map<String, Object> params) {
        params.put("companyId",AuthUtils.getCompanyId());
        List<AssetTenantUser> list = this.baseMapper.selectAssetTenantUserList(page, params);
        if(list != null && list.size()>0){
            for(AssetTenantUser tenantUser : list){
                EntityWrapper<AssetUser> wrapper = new EntityWrapper<>();
                wrapper.eq("user_id",tenantUser.getUserId());
                List<AssetUser> assetList = assetUserMapper.selectList(wrapper);
                if(assetList != null && assetList.size()>0){
                    tenantUser.setAssetList(assetList);
                }
            }
        }
        return page.setRecords(list);
    }

    @Override
    public AssetTenantUser getAssetTenantUser(Long userId) {
        AssetTenantUser tenantUser = this.baseMapper.selectById(userId);
        if(tenantUser == null){
            throw new BaseException("账号不存在");
        }
        EntityWrapper<AssetUser> wrapper = new EntityWrapper<>();
        wrapper.eq("user_id",userId);
        List<AssetUser> assetList = assetUserMapper.selectList(wrapper);
        if(assetList != null && assetList.size()>0){
            tenantUser.setAssetList(assetList);
        }

        return tenantUser;
    }

    @Override
    public void updateAssetTenantUser(AssetTenantUser assetTenantUser) {
        if(assetTenantUser.getUserId() == null){
            throw new BaseException("未获取到账户Id");
        }
        AssetTenantUser oldUser = this.baseMapper.selectById(assetTenantUser.getUserId());
        if(oldUser == null){
            throw new BaseException("账号不存在");
        }

        if(!(oldUser.getUserAccount().equals(assetTenantUser.getUserAccount()))){
            //判断账号是否唯一
            EntityWrapper<AssetTenantUser> wrapper = new EntityWrapper<>();
            wrapper.eq("company_id",AuthUtils.getCompanyId());
            wrapper.eq("user_account",assetTenantUser.getUserAccount());
            int num = this.baseMapper.selectCount(wrapper);
            if(num>0){
                throw new BaseException("账号已存在");
            }
        }

        assetTenantUser.setOperator(AuthUtils.getUserId());
        assetTenantUser.setOperateDate(new Date());

        this.baseMapper.updateById(assetTenantUser);

    }

    @Override
    public void deleteAssetTenantUser(Long userId) {
        AssetTenantUser tenantUser = this.baseMapper.selectById(userId);
        if(tenantUser == null){
            throw new BaseException("账号不存在");
        }
        AssetTenantUser updateUser = new AssetTenantUser();
        updateUser.setUserId(userId);
        updateUser.setOperator(AuthUtils.getUserId());
        updateUser.setOperateDate(new Date());
        this.baseMapper.updateById(updateUser);

        //逻辑删除账号
        this.deleteById(userId);

    }


}
