package com.ebei.device.asset.weixin.service.impl;

import com.alibaba.fastjson.JSON;
import com.ebei.device.asset.weixin.auth.AuthUser;
import com.ebei.device.asset.weixin.auth.AuthUtils;
import com.ebei.device.asset.weixin.config.CommProperties;
import com.ebei.device.asset.weixin.model.entity.AssetTenantUser;
import com.ebei.device.asset.weixin.service.AssetTenantUserService;
import com.ebei.device.asset.weixin.service.UserSessionManager;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * Redis存储用户登录授权信息
 * @author baihe
 *
 */
@Slf4j
@Service
public class RedisUserSessionManager implements UserSessionManager {

	@Autowired
    StringRedisTemplate redisTemplate;
	@Autowired
    RestTemplate restTemplate;
	@Autowired
	private AssetTenantUserService assetTenantUserService;
	@Autowired
	private CommProperties commProperties;

	private AuthUser createAuthUser(AssetTenantUser user, Long companyId) {
		user.setPwd(null);
		AuthUser auser = new AuthUser();
		auser.setUser(user);
		auser.setUserId(user.getUserId());
		auser.setUserName(user.getUserName());
		auser.setCompanyId(companyId);
		auser.setAreaId(user.getAreaId());
		auser.setTenantId(user.getTenantId());
		List<Long> assetIdList = null;
		//TODO
		assetIdList = new ArrayList<>();
		auser.setAssetIdList(assetIdList);
		return auser;
	}
	@Override
	public AuthUser createToken(AssetTenantUser user, Long companyId, String token) {
		AuthUser auser = this.createAuthUser(user,companyId);
		auser.setToken(token);
		redisTemplate.opsForValue().set(AuthUtils.REQ_TOKEN+auser.getToken(), JSON.toJSON(auser).toString());
		redisTemplate.expire(AuthUtils.REQ_TOKEN+auser.getToken(), AuthUtils.TOKEN_EXPIRE_TIME, TimeUnit.SECONDS);

		return auser;
	}
	@Override
	public String createToken(AssetTenantUser user, Long companyId) {
		String token = user.getToken();

		if(token==null) {
			token = UUID.randomUUID().toString().replaceAll("-","").toUpperCase();
			user.setToken(token);
			AssetTenantUser entity = new AssetTenantUser();
			entity.setToken(token);
			entity.setUserId(user.getUserId());
			entity.setLastLoginDate(new Date());
			assetTenantUserService.updateById(entity);
		}
		this.createToken(user,companyId,token);
		return token;
	}

	@Override
	public AuthUser getAuthUser(String token) {
		String value = redisTemplate.opsForValue().get(AuthUtils.REQ_TOKEN+token);
		if(StringUtils.isBlank(value)) {
			return null;
		}
		AuthUser user = JSON.toJavaObject(JSON.parseObject(value),AuthUser.class);
		if(user!=null) {
			redisTemplate.expire(AuthUtils.REQ_TOKEN+token, AuthUtils.TOKEN_EXPIRE_TIME, TimeUnit.SECONDS);
		}
		return user;
	}
	
	
	@Override
	public void clearToken(String token,boolean flag) {
		redisTemplate.delete(AuthUtils.REQ_TOKEN+token);
	}

	
}
