package com.ebei.device.asset.weixin.util;

import com.ebei.device.asset.weixin.auth.AuthUtils;
import com.ebei.device.asset.weixin.config.CommProperties;
import com.ebei.device.asset.weixin.constant.Constants;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpResponse;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;
import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


/**
 * Description:文件上传工具类
 *
 * @author xu
 * @date:2018/5/29
 * @Copyright (c) 2018 一碑科技
 */
public class FileUploadDownloadUtil {


    protected final static Logger logger = LoggerFactory.getLogger(FileUploadDownloadUtil.class);
       
    /**
     * 应该是用此方法
     * @param uuid
     * @param file
     * @param dateFormat
     * @param filename
     * @return
     */
    public static boolean commonFileUpload(String uuid,InputStream file,SimpleDateFormat dateFormat,String filename){
    	CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
    	
    	if(commProperties.getFileServer().contains("huiyun")) {//中通服文件服务器
    		ByteArrayOutputStream bo = new ByteArrayOutputStream();
            int bytes = 0;
            byte[] bufferOut = new byte[1024];
            try {
				while ((bytes = file.read(bufferOut)) != -1) {
					bo.write(bufferOut, 0, bytes);
				}
				file.close();
			} catch (IOException e) {
				logger.error("上传文件失败:",e);
				return false;
			}
    		return fileUploadZTF(uuid,bo.toByteArray(),dateFormat,filename);
    	}else {
    		return fileUpload(uuid,file,dateFormat,filename);
    	}
    }
    /**
    *
    * Discription : 上传文件到服务器
    * @param uuid uuid
    * @param file 文件流
    * @param dateFormat 时间格式
    * @param filename 文件名称
    * @return
    * boolean
    * @throws
    * @author : Suyc
    */
   public static boolean fileUpload(String uuid,InputStream file,SimpleDateFormat dateFormat,String filename){
	   OutputStream out = null;
	   BufferedReader reader = null;
       try {
       	   CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
       	
           HttpURLConnection conn = HttpRequestUtils.getUrlConnection(commProperties.getFileServer(), Constants.BUFFER_SIZE);
           conn.setRequestProperty("type", "1");//说明是上传
           conn.setRequestProperty("companyid", AuthUtils.getCompanyId().toString());
           conn.setRequestProperty("source", Constants.SYSTEM);//项目
           conn.setRequestProperty("range", String.valueOf(0));
           conn.setRequestProperty("filename", filename);
           conn.setRequestProperty("filecreatetime", dateFormat.format(new Date()));
           conn.setRequestProperty("fileid", uuid);
           conn.setRequestProperty("totalsize", String.valueOf(10240));

           String BOUNDARY = "---------------------------123821742118716"; //boundary就是request头和上传文件内容的分隔符
           String inputName = "file";
           String contentType = "multipart/form-data";


           StringBuffer strBuf = new StringBuffer();
           strBuf.append("\r\n").append("--").append(BOUNDARY).append("\r\n");
           strBuf.append("Content-Disposition: form-data; name=\"" + inputName + "\"; filename=\"" + filename + "\"\r\n");
           strBuf.append("Content-Type:" + contentType + "\r\n\r\n");

           out = new DataOutputStream(conn.getOutputStream());
           out.write(strBuf.toString().getBytes());
           
           int bytes = 0;
           byte[] bufferOut = new byte[1024];
           while ((bytes = file.read(bufferOut)) != -1) {
               out.write(bufferOut, 0, bytes);
           }
          
          
           byte[] endData = ("\r\n--" + BOUNDARY + "--\r\n").getBytes();
           out.write(endData);
           out.flush();
          
           //读取返回数据
           reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
           String line = null;
           StringBuilder xml = new StringBuilder();
           while ((line = reader.readLine()) != null) {
           	xml.append(line);
           }
           logger.debug("[上传文件] [{}]",xml);
           if(xml.toString().contains("<resultCode>0</resultCode>")) {
        	   return true;
           }
       } catch (Exception e) {
           logger.error("[上传文件失败]",e);
       }finally {
    	   try {
    		   if(file!=null) {
    			   file.close();
    		   }
    	   }catch (Exception e) {}
    	   try {
    		   if(out!=null) {
    			   out.close();
    		   }
    	   }catch (Exception e) {}
    	   try {
    		   if(reader!=null) {
    			   reader.close();
    		   }
    	   }catch (Exception e) {}
       }
       return false;
   }
   /**
    *
    * Discription : 通过普通方法将文件上传，并不通过web请求为源头的方式 上传文件到服务器
    * @param uuid uuid
    * @param file 文件流
    * @param dateFormat 时间格式
    * @param filename 文件名称
    * @return
    * boolean
    * @throws
    * @author : Suyc
    */
   public static boolean fileUpload(String uuid,byte[] file,SimpleDateFormat dateFormat,String filename,String companyId ){
	   OutputStream out = null;
	   BufferedReader reader = null;
	   try {
		   CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
		   HttpURLConnection conn = HttpRequestUtils.getUrlConnection(commProperties.getFileServer(), Constants.BUFFER_SIZE);
		   conn.setRequestProperty("type", "1");//说明是上传
		   conn.setRequestProperty("companyid", companyId);
		   conn.setRequestProperty("source", Constants.SYSTEM);//项目
		   conn.setRequestProperty("range", String.valueOf(0));
		   conn.setRequestProperty("filename", filename);
		   conn.setRequestProperty("filecreatetime", dateFormat.format(new Date()));
		   conn.setRequestProperty("fileid", uuid);
		   conn.setRequestProperty("totalsize", String.valueOf(10240));
		   
		   String BOUNDARY = "---------------------------123821742118716"; //boundary就是request头和上传文件内容的分隔符
		   String inputName = "file";
		   String contentType = "multipart/form-data";
		   
		   
		   StringBuffer strBuf = new StringBuffer();
		   strBuf.append("\r\n").append("--").append(BOUNDARY).append("\r\n");
		   strBuf.append("Content-Disposition: form-data; name=\"" + inputName + "\"; filename=\"" + filename + "\"\r\n");
		   strBuf.append("Content-Type:" + contentType + "\r\n\r\n");
		   
		   out = new DataOutputStream(conn.getOutputStream());
		   out.write(strBuf.toString().getBytes());
		   out.write(file);
		   byte[] endData = ("\r\n--" + BOUNDARY + "--\r\n").getBytes();
		   out.write(endData);
		   out.flush();
		   
		   //读取返回数据
		   reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
		   String line = null;
		   StringBuilder xml = new StringBuilder();
		   while ((line = reader.readLine()) != null) {
			   xml.append(line);
		   }
		   logger.debug("[上传文件] [{}]",xml);
		   if(xml.toString().contains("<resultCode>0</resultCode>")) {
			   return true;
		   }
	   } catch (Exception e) {
		   logger.error("[上传文件失败]",e);
	   }finally {
		   try {
		   }catch (Exception e) {}
		   try {
			   if(out!=null) {
				   out.close();
			   }
		   }catch (Exception e) {}
		   try {
			   if(reader!=null) {
				   reader.close();
			   }
		   }catch (Exception e) {}
	   }
	   return false;
   }
    //中通服用
    public static boolean fileUploadZTF(String uuid,byte[] file,SimpleDateFormat dateFormat,String filename){
    	OutputStream out = null;
    	BufferedReader reader = null;
        try{
        	CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
        	
            HttpURLConnection conn=HttpRequestUtils.getUrlConnection(commProperties.getFileServer(), Constants.BUFFER_SIZE);
            conn.setRequestProperty("type","1");
            conn.setRequestProperty("source", "s1");
            conn.setRequestProperty("range", String.valueOf(0));
            conn.setRequestProperty("filename", filename);
            conn.setRequestProperty("filecreatetime", dateFormat.format(new Date()));
            conn.setRequestProperty("fileid", uuid);
            conn.setRequestProperty("totalsize", String.valueOf(file.length));
            conn.setRequestProperty("companyid", AuthUtils.getCompanyId().toString());

            out = new DataOutputStream(conn.getOutputStream());

            out.write(file,0,file.length);
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            reader = new BufferedReader(new InputStreamReader(conn.getInputStream(),"utf-8"));
            String line = null;
            StringBuilder xml = new StringBuilder();
            while ((line = reader.readLine()) != null) {
            	xml.append(line);
            }
            
            logger.debug("[上传文件] [{}]",xml);
            if(xml.toString().contains("<resultCode>0</resultCode>")) {
            	return true;
            }
        } catch (Exception e) {
            logger.error("上传文件失败:",e);
        }finally {
    	   try {
    		   if(out!=null) {
    			   out.close();
    		   }
    	   }catch (Exception e) {}
    	   try {
    		   if(reader!=null) {
    			   reader.close();
    		   }
    	   }catch (Exception e) {}
    	   
       }
        return false;
    }

    /**
     * 请求文件服务器下载文件，返回下载请求,包含下载code  流  name
     * @param fileId
     * @return
     */
    public static HttpResponse getFileManagerResponse(String fileId) throws Exception {
        HttpResponse res = null;
        Map<String, String> map = new HashMap<String, String>();
        map.put("type", "0");
        map.put("range", "0");
        map.put("fileid", fileId);
        map.put("blocksize", String.valueOf(1024 * 1024 * 1024));
        // 从文件系统下载文件
        CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
        res = HttpRequestUtils.getHttpResponse(commProperties.getFileServer(), map);
        return res;
    }


    public static byte[] fileMakeLocal(String fileId) {
    	InputStream is = null;
    	ByteArrayOutputStream bo = new ByteArrayOutputStream();
    	try {
            HttpResponse res = getFileManagerResponse(fileId);
            String code = res.getFirstHeader("resultCode").getValue();
            if ("0".equals(code)) {
                is = res.getEntity().getContent();
                //输出到客户端
                byte[] buff = new byte[1024];
                int length = 0;
                while((length = is.read(buff)) > 0){
               	 bo.write(buff, 0, length);
               	 bo.flush();
                }
            }else {
            	logger.warn("[文件下载失败] [{}]",fileId);
            }
    	}catch (Exception e) {
        	logger.error("[文件下载失败] ["+fileId+"]",e);
		}finally {
			try {
                if (is != null) {
                    is.close();
                    is = null;
                }
			}catch (Exception e2) {}
		}
    	 
    	return bo.toByteArray();
    }

    /**
     * 从文件服务器下载到本地，生成临时模板文件
     *
     * @param fileId
     * @param fileName
     */
    public static String fileMakeLocal(String fileId, String fileName) {
    	CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
    	File file = new File(commProperties.getFileRoot() + "/temp/" +fileId+ fileName);
    	if(file.exists()) {
    		return file.getAbsolutePath();
    	}
        InputStream is = null;
        OutputStream os = null;
        try {
            HttpResponse res = getFileManagerResponse(fileId);
            String code = res.getFirstHeader("resultCode").getValue();
            if ("0".equals(code)) {
                is = res.getEntity().getContent();
                //输出到客户端
                os = new FileOutputStream(file);
                byte[] buff = new byte[1024];
                int length = 0;
                while((length = is.read(buff)) > 0){
                    os.write(buff, 0, length);
                    os.flush();
                }
            } else {
                return null;
            }
        } catch (Exception e) {
            logger.error("[文件下载失败]",e);
            return null;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
                if (os != null) {
                    os.close();
                    os = null;
                }
            } catch (Exception e) {
            	 logger.error("文件下载失败",e);
                return null;
            }
        }
        return file.getAbsolutePath();
    }

    public static void inputStreamToFile(InputStream ins, File file) {
        try {
            OutputStream os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            ins.close();
        } catch (Exception e) {
        	 logger.error("文件流转化失败",e);
        }
    }

    /**
     * Description:
     *
     * 获取文件路径(主要是图片展示用)
     * 这里先获取ip
     */
    public static String getRemoteFilePath(HttpServletRequest request , String fileId){
        String realIp = "";
        try{
            String requestUrl = request.getRequestURL().toString();
            //去掉http
            if(requestUrl.indexOf("https://") >= 0){
                requestUrl = requestUrl.substring(8);
            }else if(requestUrl.indexOf("http://") >= 0){
                requestUrl = requestUrl.substring(7);
            }
            //截掉后面部分
            if(requestUrl.indexOf(":") >= 0){
                requestUrl = requestUrl.substring(0,requestUrl.indexOf(":"));
            }else if(requestUrl.indexOf("/") >= 0){
                requestUrl = requestUrl.substring(0,requestUrl.indexOf("/"));
            }
            realIp = "localhost".equals(requestUrl) ?  "127.0.0.1" : requestUrl;
        }catch(Exception e){
        	logger.error("获取文件地址",e);
        }
        return FileUploadDownloadUtil.getRemoteFilePath(fileId, realIp);
    }
    /**
     * Description:
     * 这边是通过fileId 和id 拼接出地址
     *
     */
    public static String getRemoteFilePath(String fileId,String ip,String fileUrl) {
        StringBuffer pathXml = new StringBuffer();
        String path = "";
        BufferedReader br = null;
        Map<String, String> map = new HashMap<String, String>();
        map.put("type", "2");
        map.put("fileid", fileId);
        map.put("realIp", ip);

        try {
            HttpResponse res = HttpRequestUtils.getHttpResponse(fileUrl, map);
            br = new BufferedReader(new InputStreamReader(res.getEntity().getContent()));
            String sTempOneLine = new String("");
            while ((sTempOneLine = br.readLine()) != null){
                pathXml.append(sTempOneLine);
            }
            logger.debug("[获取上传文件URL] [{}]",pathXml);
        } catch (Exception e) {
        	logger.error("获取文件地址",e);
        } finally{
            try {
                if (br != null)
                    br.close();
            } catch (Exception e) {
            	logger.error("[获取上传文件URL] ",e);
            }
        }
        if(pathXml.length() > 0){
            path = convertXMLtoString(pathXml.toString(),"fileWebPath");
        }else{
            path = "-1";
        }

        return path;
    }
    /**
     * Description:
     * 这边是通过fileId 和id 拼接出地址
     *
     */
    public static String getRemoteFilePath(String fileId,String ip) {
    	CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
    	return getRemoteFilePath(fileId,ip,commProperties.getFileServer());
    }

    public static String getFileSize(String fileId,String fileServer){
    	StringBuffer fileSize = new StringBuffer();
    	BufferedReader br = null;
    	Map<String, String> map = new HashMap<String, String>();
    	map.put("type", "3");
    	map.put("fileid", fileId);
    	
    	try {
    		HttpResponse res = HttpRequestUtils.getHttpResponse(fileServer, map);
    		br = new BufferedReader(new InputStreamReader(res.getEntity().getContent()));
    		String sTempOneLine = new String("");
    		while ((sTempOneLine = br.readLine()) != null){
    			fileSize.append(sTempOneLine);
    		}
    	} catch (Exception e) {
    		logger.error("",e);
    	} finally{
    		try {
    			if (br != null)
    				br.close();
    		} catch (Exception e) {
    			e.printStackTrace();
    		}
    	}
    	
    	
    	return fileSize.toString();
    }
    public static String getFileSize(String fileId){
    	CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
    	return getFileSize(fileId,commProperties.getFileServer());
    }
    /**
     * Description:把xml装换为字符串
     *
     */
    public static String convertXMLtoString(String xml,String path) {
        Document doc = null;
        Element itemImg = null;
        Element itemCode = null;
        String filePath = "-1";
        try {
            doc = DocumentHelper.parseText(xml);
            Element rootElt = doc.getRootElement();
            Iterator iterCode = rootElt.elementIterator("resultCode");
            if(iterCode.hasNext()){
                itemCode = (Element) iterCode.next();
            }
            if(itemCode!=null&&"0".equals(itemCode.getText())){
                Iterator iterImg = rootElt.elementIterator(path);
                if(iterImg.hasNext()){
                    itemImg = (Element) iterImg.next();
                    filePath = itemImg.getText();
                }
            }
        } catch (DocumentException e) {
            logger.error("[转化xml失败]",e);
        }
        return filePath;
    }

    /**
     *
     * Discription :上传文件到tomcat目录下
     * @param file	上传的文件目录
     * @param filename	文件名称
     * @param savePathDirectory	要保存到那个文件目录
     * @param uuid	生成的uuid（文件名）
     * @return
     * @throws Exception
     * String
     * @throws
     * @author : shendx
     * @date 2014-7-14 上午9:19:00
     */
    public static String uploadFileToNativeAndReturnFilePath(HttpServletRequest request, MultipartFile file, String filename, String savePathDirectory, String uuid)
            throws Exception {
        String filepath = "";
        try {
            filename = filename.toLowerCase();
            int tc = filename.lastIndexOf(("."));
            // 获取文件类型
            String fileType = filename.substring(tc + 1);
            filepath = "";
            // 获取tomcat绝对路径
            //  String tomcatPath = request.getRealPath("/");
            File descfile = null;
            File folder = new File(savePathDirectory);
            // 查询是否有该目录,没有则创建
            if (!folder.exists()) {
                folder.mkdirs();
            }
            filepath = savePathDirectory + File.separator + uuid + "." + fileType;
            descfile = new File( filepath);
            // 上传文件主要路基
            copy(file, descfile);
        } catch (Exception e) {
            filepath = "";
        }
        return filepath;
    }

    /**
     * Copy file for file upload
     * @param src
     * @param dst
     * @throws Exception
     */
    public static void copy(MultipartFile src, File dst) throws Exception  {
        try {
            InputStream in = null ;
            OutputStream out = null ;
            try {
                in = new BufferedInputStream( src.getInputStream(), Constants.BUFFER_SIZE);
                out = new BufferedOutputStream( new FileOutputStream(dst), Constants.BUFFER_SIZE);
                byte [] buffer = new byte [Constants.BUFFER_SIZE];
                while (in.read(buffer) !=-1 ) {
                    out.write(buffer);
                    out.flush();
                    if(in.available()<Constants.BUFFER_SIZE && in.available()!=0){
                        buffer = new byte[in.available()];
                        logger.debug("buffer size: "+buffer.length);
                    }
                }
            } finally {
                if ( null != in) {
                    in.close();
                }
                if ( null != out) {
                    out.close();
                }
            }
        } catch (Exception e) {
            logger.error("[文件复制错误]",e);
            throw e;
        }
    }

    /**
     * 删除文件，可以是单个文件或文件夹
     *
     * @param fileName 待删除的文件名
     * @return 文件删除成功返回true,否则返回false
     */
    public static boolean delete(String fileName) {
        File file = new File(fileName);
        if (!file.exists()) {
            return false;
        } else {
            if (file.isFile()) {
                return deleteFile(fileName);
            } else {
                return deleteDirectory(fileName);
            }
        }
    }

    /**
     * 删除单个文件
     *
     * @param fileName 被删除文件的文件名
     * @return 单个文件删除成功返回true,否则返回false
     */
    public static boolean deleteFile(String fileName) {
        File file = new File(fileName);
        if (file.isFile() && file.exists()) {
            file.delete();
            return true;
        } else {
            return false;
        }
    }
    /**
     * 删除目录（文件夹）以及目录下的文件
     *
     * @param dir 被删除目录的文件路径
     * @return 目录删除成功返回true,否则返回false
     */
    public static boolean deleteDirectory(String dir) {
        // 如果dir不以文件分隔符结尾，自动添加文件分隔符
        if (!dir.endsWith(File.separator)) {
            dir = dir + File.separator;
        }
        File dirFile = new File(dir);
        // 如果dir对应的文件不存在，或者不是一个目录，则退出
        if (!dirFile.exists() || !dirFile.isDirectory()) {
            return false;
        }
        boolean flag = true;
        // 删除文件夹下的所有文件(包括子目录)
        File[] files = dirFile.listFiles();
        for (int i = 0; i < files.length; i++) {
            // 删除子文件
            if (files[i].isFile()) {
                flag = deleteFile(files[i].getAbsolutePath());
                if (!flag) {
                    break;
                }
            }
            // 删除子目录
            else {
                flag = deleteDirectory(files[i].getAbsolutePath());
                if (!flag) {
                    break;
                }
            }
        }

        if (!flag) {
            return false;
        }

        // 删除当前目录
        if (dirFile.delete()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 根据路径获取id
     * @param filePath
     * @return
     */
    public static String getFileIdByPath(String filePath)
    {
        String name = "";
        if(StringUtils.isNotEmpty(filePath)){
            name = filePath .substring(filePath.lastIndexOf("/")+1,filePath.lastIndexOf("."));
        }

        return name;
    }

    /**
     * 根据路径获取名字
     * @param filePath
     * @return
     */
    public static String getFileNameByPath(String filePath)
    {
        String name = "";
        if(StringUtils.isNotEmpty(filePath)){
            name = filePath .substring(filePath.lastIndexOf("/")+1,filePath.length());
        }

        return name;
    }
    public  static String getRecordFileRelativePath(String fullPath)
	{
		if(fullPath==null || fullPath.equals("")) return fullPath;
		CommProperties commProperties = SpringBeanUtil.getBean(CommProperties.class);
		String fileServer =commProperties.getFileServer();
		String basePath=fileServer.substring(0,fileServer.indexOf("FileUploadAndDownloadServlet"));
		return fullPath.substring(basePath.length());
	}
    
    public static void cleanDir(File dir) {
    	if(dir.isDirectory()) {
    		File[] fileList = dir.listFiles();
    		for(File file:fileList) {
    			cleanDir(file);
    		}
    		dir.delete();
    	}else {
    		dir.delete();
    	}
    	
    }
}
