package com.ebei.device.asset.weixin.util;


import com.baomidou.mybatisplus.plugins.Page;
import com.ebei.device.asset.weixin.constant.ResponseConstants;

/**
 * @Author: Huangweicai
 * @date 2018-02-05 16:10
 * @Description:请求封装基础类--分页
 *              结构如下
 *              {
 *                  status:200,
 *                  message:success,
 *                  data:{
 *                      records:[],      --返回内容(集合)
 *                      totalCount:100, --总共条数
 *                      pageSize:10,    --每页条数
 *                      totalPage:10    --总共页数
 *                  }
 *              }
 *              基础类 {@link ResponseEx}
 * modify :修改集成ResponseEx，那么在异常处理的时候可以统一将返回类型设置为ResponsePageEx.createError();
 * 可以在springboot viewhandler resolver不产生类型不匹配异常
 */
public class ResponsePageEx<T> extends ResponseEx<TPageBean<T>>{

    private ResponsePageEx(int status) {
    	super(status);
    }

    private ResponsePageEx(int status, String message) {
        super(status,message);
    }

    private ResponsePageEx(int status, TPageBean<T> data) {
        this(status, "", data);
    }

    private ResponsePageEx(int status, String message,TPageBean<T> data) {
        super(status,message,data);
    }

   //分页数据
    public static <T> ResponsePageEx<T> createSuccess(Page page) {
        return createSuccess((T)page.getRecords(),page.getTotal(), page.getCurrent(), page.getSize(), page.getPages());
    }

    //一页获取全部数据，没用到
    public static <T> ResponsePageEx<T> createSuccess(T t, String message,Integer totalCount) {
        return new ResponsePageEx(ResponseConstants.CODE_SUCCESS,message, TPageBean.createSuccess(t, totalCount));
    }
    //用的最多
    public static <T> ResponsePageEx<T> createSuccess(T t, Integer totalCount,Integer current,Integer size,Integer pages) {
        return createSuccess(t, "", totalCount, current, size, pages);
    }

    //分页数据
    public static <T> ResponsePageEx<T> createSuccess(T t, String message,Integer totalCount,Integer current,Integer size,Integer pages) {
        return new ResponsePageEx(ResponseConstants.CODE_SUCCESS,message, TPageBean.createSuccess(t, totalCount, current, size, pages));
    }

    public static ResponsePageEx createError(String message) {
        return createError(ResponseConstants.CODE_ERROR, message);
    }

    public static ResponsePageEx createException(String message) {
        return createError(ResponseConstants.CODE_BUSINESS_ERROR, message);
    }

    public static ResponsePageEx createError(int status,String message) {
        return new ResponsePageEx(status,message);
    }



}
